
#include <stdio.h>
#include <assert.h>
#include <sys/types.h>

#include "ttterm.h"
#include "ttbuffer.h"
#include "ttutils.h"

#include "cmdargs.h"

struct optstruct gv_opts;
extern int optind;

static void usage(const char *program_name)
{
	fprintf(stderr, "Usage: %s [options] <spec.lll> ...\n", program_name);
	fprintf(stderr, "Options:\n");
	fprintf(stderr,
		"  -h            Show usage\n");
	fprintf(stderr,
		"  -c <ident>    Place a BTYACC cut (YYVALID) on <ident>\n");
	fprintf(stderr,
		"  -s <ident>    Skeleton grammar start symbol\n");
	fprintf(stderr,
		"  -r <ident>    Identifier for rejects\n");
	fprintf(stderr,
		"  -f <format>   Output Format: use -f help to get a list\n");
	fprintf(stderr,
		"  -p <prefix>   Prefix for generated names, default: FOO\n");
	fprintf(stderr,
		"  -q <module>   Default module name, default: Junk\n");
	fprintf(stderr,
		"  -e <module>   External module\n");
	fprintf(stderr,
		"  -y <option>   Yacc option, e.g. -y \"%%glr-parser\"\n");
	fprintf(stderr,
		"  -m            Generate multiple SDF files\n");
}

static void show_formats(void)
{
	fprintf(stderr, "  c\n");
	fprintf(stderr, "  sdf\n");
	fprintf(stderr, "  html\n");
	fprintf(stderr, "  yacc\n");
	fprintf(stderr, "  haskell\n");
}

extern int getopt(int, char *const *, const char *);
extern char *optarg;

void getoptions(int argc, char *argv[])
{
	TTterm hd, tl;
	const char * str;
	int c;
	int errflg = 0;
	gv_opts.cuts = TTbuild_nil();
	gv_opts.roots = TTbuild_nil();
	gv_opts.rejects = TTbuild_nil();
	gv_opts.includes = TTbuild_nil();
	gv_opts.external_modules = TTbuild_nil();
	gv_opts.yacc_options = TTbuild_nil();
	gv_opts.multifile = FALSE;

	strcpy(gv_opts.output_format, "unknown");
	strcpy(gv_opts.prefix, "FOO");
	strcpy(gv_opts.module_prefix, "Junk");

	while (optind < argc && argv[optind][0] != '-') {
		gv_opts.includes =
			TTbuild_cons(TTbuild_string(argv[optind++]),
				gv_opts.includes);
	}

	while ((c = getopt(argc, argv, "mhc:f:r:s:p:e:y:q:")) != EOF) {
		switch (c) {
		case 'c':
			gv_opts.cuts =
			    TTbuild_cons(TTbuild_string(optarg),
				       gv_opts.cuts);
			break;
		case 's':
			gv_opts.roots =
			    TTbuild_cons(TTbuild_string(optarg),
				       gv_opts.roots);
			break;
		case 'r':
			gv_opts.rejects =
			    TTbuild_cons(TTbuild_string(optarg),
				       gv_opts.rejects);
			break;
		case 'f':
			strcpy(gv_opts.output_format, optarg);
			if (strcmp(optarg, "help") == 0) {
				show_formats();
				exit(0);
			}
			break;
		case 'p':
			strcpy(gv_opts.prefix, optarg);
			break;
		case 'q':
			strcpy(gv_opts.module_prefix, optarg);
			break;
		case 'e':
			gv_opts.external_modules =
			    TTbuild_cons(TTbuild_string(optarg),
				       gv_opts.external_modules);
			break;
		case 'y':
			gv_opts.yacc_options =
			    TTbuild_cons(TTbuild_string(optarg),
				       gv_opts.yacc_options);
			break;
		case 'm':
			gv_opts.multifile = TRUE;
			break;
		case 'h':
			usage(argv[0]);
			exit(0);
			break;
		case '?':
			errflg++;
		}
	}

	if (errflg || optind > argc) {
		usage(argv[0]);
		exit(2);
	}

	while (optind < argc) {
		gv_opts.includes =
			TTbuild_cons(TTbuild_string(argv[optind++]),
				gv_opts.includes);
	}

	if (TTmatch_nil(gv_opts.includes)) {
		usage(argv[0]);
		exit(2);
	}

	gv_opts.includes = TTreverse(gv_opts.includes);

	assert(TTmatch_cons(gv_opts.includes, &hd, &tl));
	gv_opts.includes = tl;
	assert(TTmatch_string(hd, &str));
	strcpy(gv_opts.spec, str);
}


