
#include <stdio.h>
#include <assert.h>
#include <ctype.h>
#include <string.h>
#include <sys/types.h>

#include "ttterm.h"
#include "ttbuffer.h"
#include "ttutils.h"
#include "ttfile.h"

#include "LLLParse.h"
#include "lllutils.h"
#include "lllprep.h"
#include "utils.h"

#include "genlex.h"
#include "cmdargs.h"

static FILE *f;
static TTterm st_lexicals;
static TTterm st_literals;
static const char * st_prefix;

#define BUFSZ 1024
extern int snprintf(char *s,  size_t  n,  const  char  *format, ...);
static char st_buf[BUFSZ];

/* Generate header (Scan.l). */
static void gen_header(void)
{
	fprintf(f, "%%{\n");
	fprintf(f, "#include \"ttterm.h\"\n");
	fprintf(f, "#include \"ttbuffer.h\"\n");
	fprintf(f, "#include \"ttutils.h\"\n");
	fprintf(f, "#include \"ttparse.h\"\n");
	fprintf(f, "#include \"%sFuns.h\"\n", st_prefix);

	fprintf(f, "\n");
	fprintf(f, "static void build_token(void)\n");
	fprintf(f, "{\n");
	fprintf(f, "\tyylval = TTparse_word(yyleng);\n");
	fprintf(f, "}\n");
	fprintf(f, "\n");
	fprintf(f, "%%}\n");
	fprintf(f, "\n");
	fprintf(f, "%%x PIC\n");
	fprintf(f, "%%option noyywrap\n");
	fprintf(f, "WS [ \\t\\n]\n");
	fprintf(f, "COM (%%.*\\n)\n");
	fprintf(f, "\n");
	fprintf(f, "%%%%\n");
}

/* Generate lexicals (Parse.y). */
static void gen_lexical(TTterm t)
{
	fprintf(f, "TODO {\n");
	fprintf(f, "\tbuild_token();\n");
	fprintf(f, "\tyylval = ");
	LLLdump_c_build_name(f, t);
	fprintf(f, "(yylval);\n");
	fprintf(f, "\treturn ");
	LLLdump_flex_lexical(f, t);
	fprintf(f, ";\n}\n\n");
}

static void gen_literal(TTterm t)
{
	LLLdump_literal(f, t);
	fprintf(f, " { build_token(); return ");
	LLLdump_flex_literal(f, t);
	fprintf(f, "; }\n");
}

static void gen_l(void)
{
	gen_header();
	fprintf(f, "\n");
	fprintf(f, "({WS}|{COM})+ { yylval == TTbuild_string(yytext); return FLEX_LAYOUT; }\n");
	fprintf(f, "\n");
	TTmap_void(gen_literal, st_literals);
	fprintf(f, "\n");
	TTmap_void(gen_lexical, st_lexicals);
	fprintf(f, "\n");
	fprintf(f, "%%%%\n");
}

static void gen_h(TTterm lexicals)
{
	fprintf(f, "\n");
	fprintf(f, "#ifndef YYSTYPE\n");
	fprintf(f, "#define YYSTYPE TTterm\n");
	fprintf(f, "#endif\n");
	fprintf(f, "extern YYSTYPE yylval;\n");
	fprintf(f, "\n");
	gen_flex_enum(f, "#define ", lexicals);
}

static void init(FILE * g, TTterm spec)
{
	st_prefix = gv_opts.prefix;
	st_lexicals = LLLget_undef_idents(spec);
	st_literals = LLLget_literals(spec);
	f = g;
}

void generate_lex(TTterm spec)
{
	TTterm rules, macros, expanded_spec, lexicals;

	init(f, spec);

	snprintf(st_buf, BUFSZ, "%sScan.l", gv_opts.prefix);
	f = TTfile_open(st_buf, "w");

	gen_l();

	fclose(f);

	snprintf(st_buf, BUFSZ, "%sScan.h", gv_opts.prefix);
	f = TTfile_open(st_buf, "w");

	LLLprep_yacc(spec, &rules, &macros);
	expanded_spec = TTbuildLLL_specification(TTconcat(rules, macros));
	lexicals = LLLget_undef_idents(expanded_spec);
	gen_h(lexicals);

	fclose(f);
}

