
#include <assert.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "ttterm.h"
#include "ttbuffer.h"
#include "ttutils.h"

#include "LLLParse.h"

#include "cmdargs.h"
#include "gensdf.h"
#include "lllutils.h"
#include "lllprep.h"
#include "parselll.h"
#include "utils.h"
#include "genfuns.h"
#include "genparse.h"
#include "genyacc.h"
#include "genlex.h"
#include "genhtml.h"
#include "genhaskell.h"

#define MAX_STR_SIZE 1000

#define BUFSZ 1024
extern int snprintf(char *s,  size_t  n,  const  char  *format, ...);
static TTbuf st_buf;


static BOOL formatis(const char *format)
{
	return strcmp(gv_opts.output_format, format) == 0;
}

int main(int argc, char **argv)
{
	TTbuf buf;
	TTterm spec;

	TTutils_init();
	LLLinit();

	getoptions(argc, argv);

	buf = TTbuffer_create();
	TTbuffer_read_named(buf, gv_opts.spec);
	spec = LLLparse(buf);
	TTbuffer_destroy(buf);

	spec = TTunsandwich_rec(spec);
	spec = LLLforce_qualification(spec, gv_opts.module_prefix);
	spec = closure(spec, gv_opts.roots);
	/* consistencyCheck(spec); */

	spec = LLLsimplify(spec);
	spec = LLLsort(spec);
	if (!formatis("sdf") &&
	    !formatis("html")) {
		spec = LLLprep_flatten(spec);
		LLLcheck(gv_opts.spec, spec);
	}
	
	if (formatis("c")) {
		generate_c(spec);
	} else if (formatis("sdf")) {
		generate_sdf(spec);
	} else if (formatis("html")) {
		generate_html(spec);
	} else if (formatis("yacc")) {
		generate_yacc(spec);
	} else if (formatis("haskell")) {
		generate_haskell(spec);
	} else {
		fprintf(stderr, "Unknown output format \"%s\".",
			gv_opts.output_format);
	}
	return 0;
}
