
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <ctype.h>
#include <sys/types.h>

#include "ttterm.h"
#include "ttbuffer.h"
#include "ttutils.h"

#include "LLLParse.h"
#include "lllutils.h"
#include "cmdargs.h"

#include "utils.h"

static TTterm st_closure;
static BOOL st_new_element;
static FILE * f;
static const char * st_enum_prefix;
static int st_count;

static BOOL propagate(TTterm t)
{
	TTterm str;

	if (TTmatchLLL_ident(t, &str)) {
		if (!TTelem(str, st_closure)) {
			st_closure = TTbuild_cons(str, st_closure);
			st_new_element = TRUE;
		}
		return TRUE;
	}
	return FALSE;
}

static BOOL in_closure(TTterm t)
{
	TTterm id, u, str;

	assert(TTmatchLLL_rule(t, &id, &u, &u, &u));
	assert(TTmatchLLL_ident(id, &str));
	return TTelem(str, st_closure);
}

TTterm closure(TTterm spec, TTterm roots)
{
	TTterm rules, id, u, dis, xs, ys, str, rule, root;
	BOOL root_exists;

	if (TTmatch_nil(roots)) return spec;

	assert(TTmatchLLL_specification(spec, &rules));

	xs = roots;
	while (TTmatch_cons(xs, &root, &xs)) {
		root_exists = FALSE;
		ys = rules;
		while (TTmatch_cons(ys, &rule, &ys)) {
			assert(TTmatchLLL_rule(rule, &id, &u, &dis, &u));
			assert(TTmatchLLL_ident(id, &str));
			if (TTequal(str, root)) {
				root_exists = TRUE;
			}
		}
		if (!root_exists) {
			fprintf(stderr, "error: root doesn't exists:");
			TTdump_string(stderr, root);
			fprintf(stderr, "\n");
			exit(1);
		}
	}

	st_closure = roots;
	st_new_element = TRUE;
	while (st_new_element) {
		st_new_element = FALSE;
		xs = rules;
		while (TTmatch_cons(xs, &rule, &xs)) {
			assert(TTmatchLLL_rule(rule, &id, &u, &dis, &u));
			assert(TTmatchLLL_ident(id, &str));
			if (TTelem(str, st_closure)) {
				TTtraverse(dis, propagate);
			}
		}
	}
	rules = TTfilter(in_closure, rules);
	return TTbuildLLL_specification(rules);
}

void gen_includes(FILE * g)
{
	TTterm xs, hd;
	const char * str;

	xs = gv_opts.external_modules;
	while (TTmatch_cons(xs, &hd, &xs)) {
		assert(TTmatch_string(hd, &str));
		fprintf(g, "#include \"%sScan.h\"\n", str);
		fprintf(g, "#include \"%sParse.h\"\n", str);
	}
}

static void gen_lexical(TTterm t)
{
	fprintf(f, "%s ", st_enum_prefix);
	LLLdump_flex_lexical(f, t);
	fprintf(f, " %d\n", st_count++);
}

void gen_flex_enum(FILE * g, const char * enum_prefix, TTterm lexicals)
{
	f = g;
	st_count = 257;
	st_enum_prefix = enum_prefix;
	fprintf(f, "%s FLEX_LAYOUT %d\n", st_enum_prefix, st_count++);
	TTmap_void(gen_lexical, lexicals);
}

