#include <stdio.h>
#include <assert.h>
#include <sys/types.h>
#include "ttterm.h"
#include "ttbuffer.h"
#include "ttutils.h"
#include "ttparse.h"
#include "FSTScan.h"
#include "FSTParse.h"
#include "LLLScan.h"
#include "LLLParse.h"

typedef TTterm(*fparse)(void);

static int init = 0;
static TTterm args[1000];


static TTterm TTparset_qpercentwrite(void);
static TTterm TTparset_qpercentunify(void);
static TTterm TTparset_qpercentunfold(void);
static TTterm TTparseFST_trafo_star(void);
static TTterm TTparset_qpercentseparate(void);
static TTterm TTparset_qpercentrestrict(void);
static TTterm TTparset_qpercentresolve(void);
static TTterm TTparset_qpercentrename(void);
static TTterm TTparset_qpercentreject(void);
static TTterm TTparset_qpercentto(void);
static TTterm TTparset_qpercentredefine(void);
static TTterm TTparset_qpercentpreserve(void);
static TTterm TTparset_qpercentintroduce(void);
static TTterm TTparset_qpercentinclude(void);
static TTterm TTparset_qpercentgeneralize(void);
static TTterm TTparset_qpercentfold(void);
static TTterm TTparset_qpercentexclude(void);
static TTterm TTparset_qpercenteliminate(void);
static TTterm TTparset_qpercentdelete(void);

TTterm
TTparseFST_delete( void )
{
    fparse fs[2];

    fs[0] = TTparset_qpercentdelete;
    fs[1] = TTparseLLL_ident;
    return TTparseFST_CONJUNCTION(TTfunFST_delete, fs);
}

TTterm
TTparseFST_eliminate( void )
{
    fparse fs[2];

    fs[0] = TTparset_qpercenteliminate;
    fs[1] = TTparseLLL_ident;
    return TTparseFST_CONJUNCTION(TTfunFST_eliminate, fs);
}

TTterm
TTparseFST_exclude( void )
{
    fparse fs[2];

    fs[0] = TTparset_qpercentexclude;
    fs[1] = TTparseLLL_rule;
    return TTparseFST_CONJUNCTION(TTfunFST_exclude, fs);
}

TTterm
TTparseFST_fold( void )
{
    fparse fs[2];

    fs[0] = TTparset_qpercentfold;
    fs[1] = TTparseLLL_rule;
    return TTparseFST_CONJUNCTION(TTfunFST_fold, fs);
}

TTterm
TTparseFST_generalize( void )
{
    fparse fs[2];

    fs[0] = TTparset_qpercentgeneralize;
    fs[1] = TTparseLLL_rule;
    return TTparseFST_CONJUNCTION(TTfunFST_generalize, fs);
}

TTterm
TTparseFST_include( void )
{
    fparse fs[2];

    fs[0] = TTparset_qpercentinclude;
    fs[1] = TTparseLLL_rule;
    return TTparseFST_CONJUNCTION(TTfunFST_include, fs);
}

TTterm
TTparseFST_introduce( void )
{
    fparse fs[2];

    fs[0] = TTparset_qpercentintroduce;
    fs[1] = TTparseLLL_rule;
    return TTparseFST_CONJUNCTION(TTfunFST_introduce, fs);
}

TTterm
TTparseFST_preserve( void )
{
    fparse fs[2];

    fs[0] = TTparset_qpercentpreserve;
    fs[1] = TTparseLLL_rule;
    return TTparseFST_CONJUNCTION(TTfunFST_preserve, fs);
}

TTterm
TTparseFST_redefine( void )
{
    fparse fs[4];

    fs[0] = TTparset_qpercentredefine;
    fs[1] = TTparseLLL_rule;
    fs[2] = TTparset_qpercentto;
    fs[3] = TTparseLLL_rule;
    return TTparseFST_CONJUNCTION(TTfunFST_redefine, fs);
}

TTterm
TTparseFST_reject( void )
{
    fparse fs[2];

    fs[0] = TTparset_qpercentreject;
    fs[1] = TTparseLLL_ident;
    return TTparseFST_CONJUNCTION(TTfunFST_reject, fs);
}

TTterm
TTparseFST_rename( void )
{
    fparse fs[4];

    fs[0] = TTparset_qpercentrename;
    fs[1] = TTparseLLL_ident;
    fs[2] = TTparset_qpercentto;
    fs[3] = TTparseLLL_ident;
    return TTparseFST_CONJUNCTION(TTfunFST_rename, fs);
}

TTterm
TTparseFST_resolve( void )
{
    fparse fs[2];

    fs[0] = TTparset_qpercentresolve;
    fs[1] = TTparseLLL_rule;
    return TTparseFST_CONJUNCTION(TTfunFST_resolve, fs);
}

TTterm
TTparseFST_restrict( void )
{
    fparse fs[2];

    fs[0] = TTparset_qpercentrestrict;
    fs[1] = TTparseLLL_rule;
    return TTparseFST_CONJUNCTION(TTfunFST_restrict, fs);
}

TTterm
TTparseFST_separate( void )
{
    fparse fs[4];

    fs[0] = TTparset_qpercentseparate;
    fs[1] = TTparseLLL_ident;
    fs[2] = TTparset_qpercentto;
    fs[3] = TTparseLLL_ident;
    return TTparseFST_CONJUNCTION(TTfunFST_separate, fs);
}

TTterm
TTparseFST_trafo( void )
{
    fparse fs[17];
    fs[0] = TTparseFST_delete;
    fs[1] = TTparseFST_eliminate;
    fs[2] = TTparseFST_reject;
    fs[3] = TTparseFST_rename;
    fs[4] = TTparseFST_unify;
    fs[5] = TTparseFST_separate;
    fs[6] = TTparseFST_introduce;
    fs[7] = TTparseFST_include;
    fs[8] = TTparseFST_exclude;
    fs[9] = TTparseFST_resolve;
    fs[10] = TTparseFST_preserve;
    fs[11] = TTparseFST_restrict;
    fs[12] = TTparseFST_generalize;
    fs[13] = TTparseFST_redefine;
    fs[14] = TTparseFST_fold;
    fs[15] = TTparseFST_unfold;
    fs[16] = TTparseFST_write;
    return TTparseFST_DISJUNCTION(TTfunFST_trafo, 17, fs);
}

TTterm
TTparseFST_trafos( void )
{
    fparse fs[1];

    fs[0] = TTparseFST_trafo_star;
    return TTparseFST_CONJUNCTION(TTfunFST_trafos, fs);
}

TTterm
TTparseFST_unfold( void )
{
    fparse fs[2];

    fs[0] = TTparset_qpercentunfold;
    fs[1] = TTparseLLL_ident;
    return TTparseFST_CONJUNCTION(TTfunFST_unfold, fs);
}

TTterm
TTparseFST_unify( void )
{
    fparse fs[4];

    fs[0] = TTparset_qpercentunify;
    fs[1] = TTparseLLL_ident;
    fs[2] = TTparset_qpercentto;
    fs[3] = TTparseLLL_ident;
    return TTparseFST_CONJUNCTION(TTfunFST_unify, fs);
}

TTterm
TTparseFST_write( void )
{
    fparse fs[2];

    fs[0] = TTparset_qpercentwrite;
    fs[1] = TTparseLLL_literal;
    return TTparseFST_CONJUNCTION(TTfunFST_write, fs);
}

static TTterm
TTparset_qpercentwrite( void )
{
    return TTparseFST_KEYWORD("%write");
}

static TTterm
TTparset_qpercentunify( void )
{
    return TTparseFST_KEYWORD("%unify");
}

static TTterm
TTparset_qpercentunfold( void )
{
    return TTparseFST_KEYWORD("%unfold");
}

static TTterm
TTparseFST_trafo_star( void )
{
    return TTparse_star(TTparseFST_trafo);
}

static TTterm
TTparset_qpercentseparate( void )
{
    return TTparseFST_KEYWORD("%separate");
}

static TTterm
TTparset_qpercentrestrict( void )
{
    return TTparseFST_KEYWORD("%restrict");
}

static TTterm
TTparset_qpercentresolve( void )
{
    return TTparseFST_KEYWORD("%resolve");
}

static TTterm
TTparset_qpercentrename( void )
{
    return TTparseFST_KEYWORD("%rename");
}

static TTterm
TTparset_qpercentreject( void )
{
    return TTparseFST_KEYWORD("%reject");
}

static TTterm
TTparset_qpercentto( void )
{
    return TTparseFST_KEYWORD("%to");
}

static TTterm
TTparset_qpercentredefine( void )
{
    return TTparseFST_KEYWORD("%redefine");
}

static TTterm
TTparset_qpercentpreserve( void )
{
    return TTparseFST_KEYWORD("%preserve");
}

static TTterm
TTparset_qpercentintroduce( void )
{
    return TTparseFST_KEYWORD("%introduce");
}

static TTterm
TTparset_qpercentinclude( void )
{
    return TTparseFST_KEYWORD("%include");
}

static TTterm
TTparset_qpercentgeneralize( void )
{
    return TTparseFST_KEYWORD("%generalize");
}

static TTterm
TTparset_qpercentfold( void )
{
    return TTparseFST_KEYWORD("%fold");
}

static TTterm
TTparset_qpercentexclude( void )
{
    return TTparseFST_KEYWORD("%exclude");
}

static TTterm
TTparset_qpercenteliminate( void )
{
    return TTparseFST_KEYWORD("%eliminate");
}

static TTterm
TTparset_qpercentdelete( void )
{
    return TTparseFST_KEYWORD("%delete");
}


BOOL
FSTisKeyword(const char * name)
{
    return (
            strcmp(name, "%delete") == 0 ||
            strcmp(name, "%eliminate") == 0 ||
            strcmp(name, "%exclude") == 0 ||
            strcmp(name, "%fold") == 0 ||
            strcmp(name, "%generalize") == 0 ||
            strcmp(name, "%include") == 0 ||
            strcmp(name, "%introduce") == 0 ||
            strcmp(name, "%preserve") == 0 ||
            strcmp(name, "%redefine") == 0 ||
            strcmp(name, "%reject") == 0 ||
            strcmp(name, "%rename") == 0 ||
            strcmp(name, "%resolve") == 0 ||
            strcmp(name, "%restrict") == 0 ||
            strcmp(name, "%separate") == 0 ||
            strcmp(name, "%to") == 0 ||
            strcmp(name, "%unfold") == 0 ||
            strcmp(name, "%unify") == 0 ||
            strcmp(name, "%write") == 0 ||
            FALSE);
}
void
FSTinit( void )
{
    if(init == 0)
    {
        TTutils_init();
        TTfunFST_delete = TTbuild_fun("FST_delete", 2);
        TTfunFST_eliminate = TTbuild_fun("FST_eliminate", 2);
        TTfunFST_exclude = TTbuild_fun("FST_exclude", 2);
        TTfunFST_fold = TTbuild_fun("FST_fold", 2);
        TTfunFST_generalize = TTbuild_fun("FST_generalize", 2);
        TTfunFST_include = TTbuild_fun("FST_include", 2);
        TTfunFST_introduce = TTbuild_fun("FST_introduce", 2);
        TTfunFST_preserve = TTbuild_fun("FST_preserve", 2);
        TTfunFST_redefine = TTbuild_fun("FST_redefine", 4);
        TTfunFST_reject = TTbuild_fun("FST_reject", 2);
        TTfunFST_rename = TTbuild_fun("FST_rename", 4);
        TTfunFST_resolve = TTbuild_fun("FST_resolve", 2);
        TTfunFST_restrict = TTbuild_fun("FST_restrict", 2);
        TTfunFST_separate = TTbuild_fun("FST_separate", 4);
        TTfunFST_trafo = TTbuild_fun("FST_trafo", 1);
        TTfunFST_trafos = TTbuild_fun("FST_trafos", 1);
        TTfunFST_unfold = TTbuild_fun("FST_unfold", 2);
        TTfunFST_unify = TTbuild_fun("FST_unify", 4);
        TTfunFST_write = TTbuild_fun("FST_write", 2);
    }
    init++;
}

static BOOL TTist_qpercentwrite(TTterm t);
static BOOL TTist_qpercentunify(TTterm t);
static BOOL TTist_qpercentunfold(TTterm t);
static BOOL TTisFST_trafo_star(TTterm t);
static BOOL TTist_qpercentseparate(TTterm t);
static BOOL TTist_qpercentrestrict(TTterm t);
static BOOL TTist_qpercentresolve(TTterm t);
static BOOL TTist_qpercentrename(TTterm t);
static BOOL TTist_qpercentreject(TTterm t);
static BOOL TTist_qpercentto(TTterm t);
static BOOL TTist_qpercentredefine(TTterm t);
static BOOL TTist_qpercentpreserve(TTterm t);
static BOOL TTist_qpercentintroduce(TTterm t);
static BOOL TTist_qpercentinclude(TTterm t);
static BOOL TTist_qpercentgeneralize(TTterm t);
static BOOL TTist_qpercentfold(TTterm t);
static BOOL TTist_qpercentexclude(TTterm t);
static BOOL TTist_qpercenteliminate(TTterm t);
static BOOL TTist_qpercentdelete(TTterm t);

static BOOL
TTist_qpercentwrite(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%write");
}

static BOOL
TTist_qpercentunify(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%unify");
}

static BOOL
TTist_qpercentunfold(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%unfold");
}

static BOOL
TTisFST_trafo_star(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_star(t, TTisFST_trafo);
}

static BOOL
TTist_qpercentseparate(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%separate");
}

static BOOL
TTist_qpercentrestrict(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%restrict");
}

static BOOL
TTist_qpercentresolve(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%resolve");
}

static BOOL
TTist_qpercentrename(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%rename");
}

static BOOL
TTist_qpercentreject(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%reject");
}

static BOOL
TTist_qpercentto(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%to");
}

static BOOL
TTist_qpercentredefine(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%redefine");
}

static BOOL
TTist_qpercentpreserve(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%preserve");
}

static BOOL
TTist_qpercentintroduce(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%introduce");
}

static BOOL
TTist_qpercentinclude(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%include");
}

static BOOL
TTist_qpercentgeneralize(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%generalize");
}

static BOOL
TTist_qpercentfold(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%fold");
}

static BOOL
TTist_qpercentexclude(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%exclude");
}

static BOOL
TTist_qpercenteliminate(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%eliminate");
}

static BOOL
TTist_qpercentdelete(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%delete");
}

BOOL
TTisFST_delete(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_delete;
}

BOOL
TTisFST_eliminate(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_eliminate;
}

BOOL
TTisFST_exclude(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_exclude;
}

BOOL
TTisFST_fold(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_fold;
}

BOOL
TTisFST_generalize(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_generalize;
}

BOOL
TTisFST_include(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_include;
}

BOOL
TTisFST_introduce(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_introduce;
}

BOOL
TTisFST_preserve(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_preserve;
}

BOOL
TTisFST_redefine(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_redefine;
}

BOOL
TTisFST_reject(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_reject;
}

BOOL
TTisFST_rename(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_rename;
}

BOOL
TTisFST_resolve(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_resolve;
}

BOOL
TTisFST_restrict(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_restrict;
}

BOOL
TTisFST_separate(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_separate;
}

BOOL
TTisFST_trafo(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_trafo;
}

BOOL
TTisFST_trafos(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_trafos;
}

BOOL
TTisFST_unfold(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_unfold;
}

BOOL
TTisFST_unify(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_unify;
}

BOOL
TTisFST_write(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunFST_write;
}

TTterm
TTbuildFST_delete(TTterm t1, TTterm t2)
{
	assert(TTist_qpercentdelete(t1) && TTisLLL_ident(t2));
    args[0] = t1;
    args[1] = t2;
    return TTbuild_term(TTfunFST_delete, args);
}

TTterm
TTbuildFST_eliminate(TTterm t1, TTterm t2)
{
	assert(TTist_qpercenteliminate(t1) && TTisLLL_ident(t2));
    args[0] = t1;
    args[1] = t2;
    return TTbuild_term(TTfunFST_eliminate, args);
}

TTterm
TTbuildFST_exclude(TTterm t1, TTterm t2)
{
	assert(TTist_qpercentexclude(t1) && TTisLLL_rule(t2));
    args[0] = t1;
    args[1] = t2;
    return TTbuild_term(TTfunFST_exclude, args);
}

TTterm
TTbuildFST_fold(TTterm t1, TTterm t2)
{
	assert(TTist_qpercentfold(t1) && TTisLLL_rule(t2));
    args[0] = t1;
    args[1] = t2;
    return TTbuild_term(TTfunFST_fold, args);
}

TTterm
TTbuildFST_generalize(TTterm t1, TTterm t2)
{
	assert(TTist_qpercentgeneralize(t1) && TTisLLL_rule(t2));
    args[0] = t1;
    args[1] = t2;
    return TTbuild_term(TTfunFST_generalize, args);
}

TTterm
TTbuildFST_include(TTterm t1, TTterm t2)
{
	assert(TTist_qpercentinclude(t1) && TTisLLL_rule(t2));
    args[0] = t1;
    args[1] = t2;
    return TTbuild_term(TTfunFST_include, args);
}

TTterm
TTbuildFST_introduce(TTterm t1, TTterm t2)
{
	assert(TTist_qpercentintroduce(t1) && TTisLLL_rule(t2));
    args[0] = t1;
    args[1] = t2;
    return TTbuild_term(TTfunFST_introduce, args);
}

TTterm
TTbuildFST_preserve(TTterm t1, TTterm t2)
{
	assert(TTist_qpercentpreserve(t1) && TTisLLL_rule(t2));
    args[0] = t1;
    args[1] = t2;
    return TTbuild_term(TTfunFST_preserve, args);
}

TTterm
TTbuildFST_redefine(TTterm t1, TTterm t2, TTterm t3, TTterm t4)
{
	assert(TTist_qpercentredefine(t1) && TTisLLL_rule(t2) && TTist_qpercentto(t3) && TTisLLL_rule(t4));
    args[0] = t1;
    args[1] = t2;
    args[2] = t3;
    args[3] = t4;
    return TTbuild_term(TTfunFST_redefine, args);
}

TTterm
TTbuildFST_reject(TTterm t1, TTterm t2)
{
	assert(TTist_qpercentreject(t1) && TTisLLL_ident(t2));
    args[0] = t1;
    args[1] = t2;
    return TTbuild_term(TTfunFST_reject, args);
}

TTterm
TTbuildFST_rename(TTterm t1, TTterm t2, TTterm t3, TTterm t4)
{
	assert(TTist_qpercentrename(t1) && TTisLLL_ident(t2) && TTist_qpercentto(t3) && TTisLLL_ident(t4));
    args[0] = t1;
    args[1] = t2;
    args[2] = t3;
    args[3] = t4;
    return TTbuild_term(TTfunFST_rename, args);
}

TTterm
TTbuildFST_resolve(TTterm t1, TTterm t2)
{
	assert(TTist_qpercentresolve(t1) && TTisLLL_rule(t2));
    args[0] = t1;
    args[1] = t2;
    return TTbuild_term(TTfunFST_resolve, args);
}

TTterm
TTbuildFST_restrict(TTterm t1, TTterm t2)
{
	assert(TTist_qpercentrestrict(t1) && TTisLLL_rule(t2));
    args[0] = t1;
    args[1] = t2;
    return TTbuild_term(TTfunFST_restrict, args);
}

TTterm
TTbuildFST_separate(TTterm t1, TTterm t2, TTterm t3, TTterm t4)
{
	assert(TTist_qpercentseparate(t1) && TTisLLL_ident(t2) && TTist_qpercentto(t3) && TTisLLL_ident(t4));
    args[0] = t1;
    args[1] = t2;
    args[2] = t3;
    args[3] = t4;
    return TTbuild_term(TTfunFST_separate, args);
}

TTterm
TTbuildFST_trafo(TTterm t1)
{
	assert(TTisFST_delete(t1) || TTisFST_eliminate(t1) || TTisFST_reject(t1) || TTisFST_rename(t1) || TTisFST_unify(t1) || TTisFST_separate(t1) || TTisFST_introduce(t1) || TTisFST_include(t1) || TTisFST_exclude(t1) || TTisFST_resolve(t1) || TTisFST_preserve(t1) || TTisFST_restrict(t1) || TTisFST_generalize(t1) || TTisFST_redefine(t1) || TTisFST_fold(t1) || TTisFST_unfold(t1) || TTisFST_write(t1));
    args[0] = t1;
    return TTbuild_term(TTfunFST_trafo, args);
}

TTterm
TTbuildFST_trafos(TTterm t1)
{
	assert(TTisFST_trafo_star(t1));
    args[0] = t1;
    return TTbuild_term(TTfunFST_trafos, args);
}

TTterm
TTbuildFST_unfold(TTterm t1, TTterm t2)
{
	assert(TTist_qpercentunfold(t1) && TTisLLL_ident(t2));
    args[0] = t1;
    args[1] = t2;
    return TTbuild_term(TTfunFST_unfold, args);
}

TTterm
TTbuildFST_unify(TTterm t1, TTterm t2, TTterm t3, TTterm t4)
{
	assert(TTist_qpercentunify(t1) && TTisLLL_ident(t2) && TTist_qpercentto(t3) && TTisLLL_ident(t4));
    args[0] = t1;
    args[1] = t2;
    args[2] = t3;
    args[3] = t4;
    return TTbuild_term(TTfunFST_unify, args);
}

TTterm
TTbuildFST_write(TTterm t1, TTterm t2)
{
	assert(TTist_qpercentwrite(t1) && TTisLLL_literal(t2));
    args[0] = t1;
    args[1] = t2;
    return TTbuild_term(TTfunFST_write, args);
}

BOOL
TTmatchFST_delete(TTterm t, TTterm *t1, TTterm *t2)
{
    if (t->fun == TTfunFST_delete)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_eliminate(TTterm t, TTterm *t1, TTterm *t2)
{
    if (t->fun == TTfunFST_eliminate)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_exclude(TTterm t, TTterm *t1, TTterm *t2)
{
    if (t->fun == TTfunFST_exclude)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_fold(TTterm t, TTterm *t1, TTterm *t2)
{
    if (t->fun == TTfunFST_fold)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_generalize(TTterm t, TTterm *t1, TTterm *t2)
{
    if (t->fun == TTfunFST_generalize)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_include(TTterm t, TTterm *t1, TTterm *t2)
{
    if (t->fun == TTfunFST_include)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_introduce(TTterm t, TTterm *t1, TTterm *t2)
{
    if (t->fun == TTfunFST_introduce)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_preserve(TTterm t, TTterm *t1, TTterm *t2)
{
    if (t->fun == TTfunFST_preserve)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_redefine(TTterm t, TTterm *t1, TTterm *t2, TTterm *t3, TTterm *t4)
{
    if (t->fun == TTfunFST_redefine)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        *t3 = t->u.args[2];
        *t4 = t->u.args[3];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_reject(TTterm t, TTterm *t1, TTterm *t2)
{
    if (t->fun == TTfunFST_reject)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_rename(TTterm t, TTterm *t1, TTterm *t2, TTterm *t3, TTterm *t4)
{
    if (t->fun == TTfunFST_rename)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        *t3 = t->u.args[2];
        *t4 = t->u.args[3];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_resolve(TTterm t, TTterm *t1, TTterm *t2)
{
    if (t->fun == TTfunFST_resolve)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_restrict(TTterm t, TTterm *t1, TTterm *t2)
{
    if (t->fun == TTfunFST_restrict)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_separate(TTterm t, TTterm *t1, TTterm *t2, TTterm *t3, TTterm *t4)
{
    if (t->fun == TTfunFST_separate)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        *t3 = t->u.args[2];
        *t4 = t->u.args[3];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_trafo(TTterm t, TTterm *t1)
{
    if (t->fun == TTfunFST_trafo)
    {
        *t1 = t->u.args[0];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_trafos(TTterm t, TTterm *t1)
{
    if (t->fun == TTfunFST_trafos)
    {
        *t1 = t->u.args[0];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_unfold(TTterm t, TTterm *t1, TTterm *t2)
{
    if (t->fun == TTfunFST_unfold)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_unify(TTterm t, TTterm *t1, TTterm *t2, TTterm *t3, TTterm *t4)
{
    if (t->fun == TTfunFST_unify)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        *t3 = t->u.args[2];
        *t4 = t->u.args[3];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchFST_write(TTterm t, TTterm *t1, TTterm *t2)
{
    if (t->fun == TTfunFST_write)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        return TRUE;
    }
    return FALSE;
}

