
#include <stdio.h>
#include <assert.h>
#include <sys/types.h>

#include "ttterm.h"
#include "ttbuffer.h"
#include "ttutils.h"

#include "LLLParse.h"
#include "lllpp.h"
#include "FSTParse.h"
#include "fstpp.h"

static TTbuf st_buf;

static BOOL dump(TTterm t)
{
	TTterm a, b, hd, tl;
	TTterm id, id2;
	TTterm rule, rule2;
	TTterm terms;
	const char * str;

	if (TTmatch_cons(t, &hd, &tl) &&
	    hd->fun == TTfunFST_trafo) {
		TTtraverse(hd, dump);
		if (!TTmatch_nil(tl)) {
			TTbuffer_append(st_buf, "\n\n");
			TTtraverse(tl, dump);
		}
		return TRUE;
	} else if (TTmatchFST_delete(t, &a, &id) ||
	    TTmatchFST_eliminate(t, &a, &id) ||
	    TTmatchFST_reject(t, &a, &id) ||
	    TTmatchFST_unfold(t, &a, &id)) {
	    	assert(TTmatch_string(a, &str));
		TTbuffer_append(st_buf, str);
		LLLpretty(st_buf, id);
		return TRUE;
	} else if (TTmatchFST_rename(t, &a, &id, &b, &id2) ||
		   TTmatchFST_unify(t, &a, &id, &b, &id2) ||
		   TTmatchFST_separate(t, &a, &id, &b, &id2)) {
	    	assert(TTmatch_string(a, &str));
		TTbuffer_append(st_buf, str);
		LLLpretty(st_buf, id);
		TTbuffer_append(st_buf, " ");
	    	assert(TTmatch_string(b, &str));
		TTbuffer_append(st_buf, str);
		LLLpretty(st_buf, id2);
		return TRUE;
	} else if (TTmatchFST_introduce(t, &a, &rule) ||
		   TTmatchFST_include(t, &a, &rule) ||
		   TTmatchFST_exclude(t, &a, &rule) ||
		   TTmatchFST_resolve(t, &a, &rule) ||
		   TTmatchFST_restrict(t, &a, &rule) ||
		   TTmatchFST_generalize(t, &a, &rule) ||
		   TTmatchFST_preserve(t, &a, &rule) ||
		   TTmatchFST_fold(t, &a, &rule)) {
	    	assert(TTmatch_string(a, &str));
		TTbuffer_append(st_buf, str);
		TTbuffer_append(st_buf, "\n");
		TTtraverse(rule, dump);
		return TRUE;
	} else if (TTmatchFST_redefine(t, &a, &rule, &b, &rule2)) {
	    	assert(TTmatch_string(a, &str));
		TTbuffer_append(st_buf, str);
		TTbuffer_append(st_buf, "\n");
		LLLpretty_rule(st_buf, rule);
		TTbuffer_append(st_buf, "\n");
	    	assert(TTmatch_string(b, &str));
		TTbuffer_append(st_buf, str);
		TTbuffer_append(st_buf, "\n");
		LLLpretty_rule(st_buf, rule2);
		return TRUE;
	} else if (t->fun == TTfunLLL_ident
		   || t->fun == TTfunLLL_rule) {
		LLLpretty(st_buf, t);
		return TRUE;
	}
	return FALSE;
}

void FSTpretty(TTbuf buf, TTterm t)
{
	st_buf = buf;

	TTtraverse(t, dump);
}
