/* A non-implementation for Boehm's gc interface, all operations are mapped
   to normal malloc/free operations, so no garbage is collected other than
   what the explicit GC_free collects. This motivated by two problems:
   - On some systems Boehm's garbage collector doesn't work yet.
   - Some people want a self contained package.
 */

#include <stdlib.h>
#include <assert.h>
#include "gc.h"

void * GC_malloc(size_t nbytes)
{
	void * p = malloc(nbytes);
	assert(p);
	return p;
}

void * GC_malloc_atomic(size_t nbytes)
{
	return GC_malloc(nbytes);
}

void * GC_malloc_uncollectable(size_t nbytes)
{
	return GC_malloc(nbytes);
}

void * GC_realloc(void *old, size_t new_size)
{
	void * p = realloc(old, new_size);
	assert(p);
	return p;
}

void GC_free(void *dead)
{
	free(dead);
}

void * GC_malloc_ignore_off_page(size_t nbytes)
{
	return GC_malloc(nbytes);
}

void * GC_malloc_atomic_ignore_off_page(size_t nbytes)
{
	return GC_malloc(nbytes);
}

void GC_gcollect(void)
{
}

void GC_enable_incremental(void)
{
}

GC_warn_proc GC_set_warn_proc(GC_warn_proc p)
{
	return p;
}

/* This is not legal ansi C, so can't support it. */
/*
void GC_register_finalizer(...)
{
}
*/


