#include <stdio.h>
#include <assert.h>
#include <sys/types.h>
#include "ttterm.h"
#include "ttbuffer.h"
#include "ttutils.h"
#include "ttparse.h"
#include "LLLScan.h"
#include "LLLParse.h"

typedef TTterm(*fparse)(void);

static int init = 0;
static TTterm args[1000];


static TTterm TTparseLLL_repetition_opt(void);
static TTterm TTparseLLL_rule_plus(void);
static TTterm TTparset_qsemicolon(void);
static TTterm TTparset_qcolon(void);
static TTterm TTparset_qquestion(void);
static TTterm TTparset_qmultiply(void);
static TTterm TTparset_qplus(void);
static TTterm TTparset_qpclose(void);
static TTterm TTparset_qpopen(void);
static TTterm TTparseLLL_conjunction_sepby_t_qpipe(void);
static TTterm TTparset_qpipe(void);
static TTterm TTparseLLL_term_plus(void);
static TTterm TTparset_qpercentepsilon(void);
static TTterm TTparset_qcbclose(void);
static TTterm TTparset_qcbopen(void);

TTterm
TTparseLLL_alternation( void )
{
    fparse fs[5];

    fs[0] = TTparset_qcbopen;
    fs[1] = TTparseLLL_basis;
    fs[2] = TTparseLLL_basis;
    fs[3] = TTparset_qcbclose;
    fs[4] = TTparseLLL_repetition;
    return TTparseLLL_CONJUNCTION(TTfunLLL_alternation, fs);
}

TTterm
TTparseLLL_basis( void )
{
    fparse fs[5];
    fs[0] = TTparseLLL_ident;
    fs[1] = TTparseLLL_literal;
    fs[2] = TTparset_qpercentepsilon;
    fs[3] = TTparseLLL_alternation;
    fs[4] = TTparseLLL_group;
    return TTparseLLL_DISJUNCTION(TTfunLLL_basis, 5, fs);
}

TTterm
TTparseLLL_conjunction( void )
{
    fparse fs[1];

    fs[0] = TTparseLLL_term_plus;
    return TTparseLLL_CONJUNCTION(TTfunLLL_conjunction, fs);
}

TTterm
TTparseLLL_disjunction( void )
{
    fparse fs[1];

    fs[0] = TTparseLLL_conjunction_sepby_t_qpipe;
    return TTparseLLL_CONJUNCTION(TTfunLLL_disjunction, fs);
}

TTterm
TTparseLLL_group( void )
{
    fparse fs[3];

    fs[0] = TTparset_qpopen;
    fs[1] = TTparseLLL_disjunction;
    fs[2] = TTparset_qpclose;
    return TTparseLLL_CONJUNCTION(TTfunLLL_group, fs);
}

TTterm
TTparseLLL_repetition( void )
{
    fparse fs[3];
    fs[0] = TTparset_qplus;
    fs[1] = TTparset_qmultiply;
    fs[2] = TTparset_qquestion;
    return TTparseLLL_DISJUNCTION(TTfunLLL_repetition, 3, fs);
}

TTterm
TTparseLLL_rule( void )
{
    fparse fs[4];

    fs[0] = TTparseLLL_ident;
    fs[1] = TTparset_qcolon;
    fs[2] = TTparseLLL_disjunction;
    fs[3] = TTparset_qsemicolon;
    return TTparseLLL_CONJUNCTION(TTfunLLL_rule, fs);
}

TTterm
TTparseLLL_specification( void )
{
    fparse fs[1];

    fs[0] = TTparseLLL_rule_plus;
    return TTparseLLL_CONJUNCTION(TTfunLLL_specification, fs);
}

TTterm
TTparseLLL_term( void )
{
    fparse fs[2];

    fs[0] = TTparseLLL_basis;
    fs[1] = TTparseLLL_repetition_opt;
    return TTparseLLL_CONJUNCTION(TTfunLLL_term, fs);
}

static TTterm
TTparseLLL_repetition_opt( void )
{
    return TTparse_optional(TTparseLLL_repetition);
}

static TTterm
TTparseLLL_rule_plus( void )
{
    return TTparse_plus(TTparseLLL_rule);
}

static TTterm
TTparset_qsemicolon( void )
{
    return TTparseLLL_KEYWORD(";");
}

static TTterm
TTparset_qcolon( void )
{
    return TTparseLLL_KEYWORD(":");
}

static TTterm
TTparset_qquestion( void )
{
    return TTparseLLL_KEYWORD("?");
}

static TTterm
TTparset_qmultiply( void )
{
    return TTparseLLL_KEYWORD("*");
}

static TTterm
TTparset_qplus( void )
{
    return TTparseLLL_KEYWORD("+");
}

static TTterm
TTparset_qpclose( void )
{
    return TTparseLLL_KEYWORD(")");
}

static TTterm
TTparset_qpopen( void )
{
    return TTparseLLL_KEYWORD("(");
}

static TTterm
TTparseLLL_conjunction_sepby_t_qpipe( void )
{
    return TTparse_alternation_plus(TTparseLLL_conjunction, TTparset_qpipe);
}

static TTterm
TTparset_qpipe( void )
{
    return TTparseLLL_KEYWORD("|");
}

static TTterm
TTparseLLL_term_plus( void )
{
    return TTparse_plus(TTparseLLL_term);
}

static TTterm
TTparset_qpercentepsilon( void )
{
    return TTparseLLL_KEYWORD("%epsilon");
}

static TTterm
TTparset_qcbclose( void )
{
    return TTparseLLL_KEYWORD("}");
}

static TTterm
TTparset_qcbopen( void )
{
    return TTparseLLL_KEYWORD("{");
}


BOOL
LLLisKeyword(const char * name)
{
    return (
            strcmp(name, "%epsilon") == 0 ||
            strcmp(name, "(") == 0 ||
            strcmp(name, ")") == 0 ||
            strcmp(name, "*") == 0 ||
            strcmp(name, "+") == 0 ||
            strcmp(name, ":") == 0 ||
            strcmp(name, ";") == 0 ||
            strcmp(name, "?") == 0 ||
            strcmp(name, "{") == 0 ||
            strcmp(name, "|") == 0 ||
            strcmp(name, "}") == 0 ||
            FALSE);
}
void
LLLinit( void )
{
    if(init == 0)
    {
        TTutils_init();
        TTfunLLL_alternation = TTbuild_fun("LLL_alternation", 5);
        TTfunLLL_basis = TTbuild_fun("LLL_basis", 1);
        TTfunLLL_conjunction = TTbuild_fun("LLL_conjunction", 1);
        TTfunLLL_disjunction = TTbuild_fun("LLL_disjunction", 1);
        TTfunLLL_group = TTbuild_fun("LLL_group", 3);
        TTfunLLL_repetition = TTbuild_fun("LLL_repetition", 1);
        TTfunLLL_rule = TTbuild_fun("LLL_rule", 4);
        TTfunLLL_specification = TTbuild_fun("LLL_specification", 1);
        TTfunLLL_term = TTbuild_fun("LLL_term", 2);
        TTfunLLL_literal = TTbuild_fun("LLL_literal", 1);
        TTfunLLL_ident = TTbuild_fun("LLL_ident", 1);
    }
    init++;
}

static BOOL TTisLLL_repetition_opt(TTterm t);
static BOOL TTisLLL_rule_plus(TTterm t);
static BOOL TTist_qsemicolon(TTterm t);
static BOOL TTist_qcolon(TTterm t);
static BOOL TTist_qquestion(TTterm t);
static BOOL TTist_qmultiply(TTterm t);
static BOOL TTist_qplus(TTterm t);
static BOOL TTist_qpclose(TTterm t);
static BOOL TTist_qpopen(TTterm t);
static BOOL TTisLLL_conjunction_sepby_t_qpipe(TTterm t);
static BOOL TTist_qpipe(TTterm t);
static BOOL TTisLLL_term_plus(TTterm t);
static BOOL TTist_qpercentepsilon(TTterm t);
static BOOL TTist_qcbclose(TTterm t);
static BOOL TTist_qcbopen(TTterm t);

static BOOL
TTisLLL_repetition_opt(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_optional(t, TTisLLL_repetition);
}

static BOOL
TTisLLL_rule_plus(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_plus(t, TTisLLL_rule);
}

static BOOL
TTist_qsemicolon(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, ";");
}

static BOOL
TTist_qcolon(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, ":");
}

static BOOL
TTist_qquestion(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "?");
}

static BOOL
TTist_qmultiply(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "*");
}

static BOOL
TTist_qplus(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "+");
}

static BOOL
TTist_qpclose(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, ")");
}

static BOOL
TTist_qpopen(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "(");
}

static BOOL
TTisLLL_conjunction_sepby_t_qpipe(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_plus(t, TTisLLL_conjunction);
}

static BOOL
TTist_qpipe(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "|");
}

static BOOL
TTisLLL_term_plus(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_plus(t, TTisLLL_term);
}

static BOOL
TTist_qpercentepsilon(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "%epsilon");
}

static BOOL
TTist_qcbclose(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "}");
}

static BOOL
TTist_qcbopen(TTterm t)
{
	t = TTunsandwich(t);
	return TTtypecheck_literal(t, "{");
}

BOOL
TTisLLL_alternation(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunLLL_alternation;
}

BOOL
TTisLLL_basis(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunLLL_basis;
}

BOOL
TTisLLL_conjunction(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunLLL_conjunction;
}

BOOL
TTisLLL_disjunction(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunLLL_disjunction;
}

BOOL
TTisLLL_group(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunLLL_group;
}

BOOL
TTisLLL_repetition(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunLLL_repetition;
}

BOOL
TTisLLL_rule(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunLLL_rule;
}

BOOL
TTisLLL_specification(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunLLL_specification;
}

BOOL
TTisLLL_term(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunLLL_term;
}

BOOL
TTisLLL_literal(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunLLL_literal;
}

BOOL
TTisLLL_ident(TTterm t)
{
	t = TTunsandwich(t);
	return t->fun == TTfunLLL_ident;
}

TTterm
TTbuildLLL_alternation(TTterm t1, TTterm t2, TTterm t3, TTterm t4, TTterm t5)
{
	assert(TTist_qcbopen(t1) && TTisLLL_basis(t2) && TTisLLL_basis(t3) && TTist_qcbclose(t4) && TTisLLL_repetition(t5));
    args[0] = t1;
    args[1] = t2;
    args[2] = t3;
    args[3] = t4;
    args[4] = t5;
    return TTbuild_term(TTfunLLL_alternation, args);
}

TTterm
TTbuildLLL_basis(TTterm t1)
{
	assert(TTisLLL_ident(t1) || TTisLLL_literal(t1) || TTist_qpercentepsilon(t1) || TTisLLL_alternation(t1) || TTisLLL_group(t1));
    args[0] = t1;
    return TTbuild_term(TTfunLLL_basis, args);
}

TTterm
TTbuildLLL_conjunction(TTterm t1)
{
	assert(TTisLLL_term_plus(t1));
    args[0] = t1;
    return TTbuild_term(TTfunLLL_conjunction, args);
}

TTterm
TTbuildLLL_disjunction(TTterm t1)
{
	assert(TTisLLL_conjunction_sepby_t_qpipe(t1));
    args[0] = t1;
    return TTbuild_term(TTfunLLL_disjunction, args);
}

TTterm
TTbuildLLL_group(TTterm t1, TTterm t2, TTterm t3)
{
	assert(TTist_qpopen(t1) && TTisLLL_disjunction(t2) && TTist_qpclose(t3));
    args[0] = t1;
    args[1] = t2;
    args[2] = t3;
    return TTbuild_term(TTfunLLL_group, args);
}

TTterm
TTbuildLLL_repetition(TTterm t1)
{
	assert(TTist_qplus(t1) || TTist_qmultiply(t1) || TTist_qquestion(t1));
    args[0] = t1;
    return TTbuild_term(TTfunLLL_repetition, args);
}

TTterm
TTbuildLLL_rule(TTterm t1, TTterm t2, TTterm t3, TTterm t4)
{
	assert(TTisLLL_ident(t1) && TTist_qcolon(t2) && TTisLLL_disjunction(t3) && TTist_qsemicolon(t4));
    args[0] = t1;
    args[1] = t2;
    args[2] = t3;
    args[3] = t4;
    return TTbuild_term(TTfunLLL_rule, args);
}

TTterm
TTbuildLLL_specification(TTterm t1)
{
	assert(TTisLLL_rule_plus(t1));
    args[0] = t1;
    return TTbuild_term(TTfunLLL_specification, args);
}

TTterm
TTbuildLLL_term(TTterm t1, TTterm t2)
{
	assert(TTisLLL_basis(t1) && TTisLLL_repetition_opt(t2));
    args[0] = t1;
    args[1] = t2;
    return TTbuild_term(TTfunLLL_term, args);
}

TTterm
TTbuildLLL_literal(TTtype_string t)
{
    args[0] = (TTterm)t;
    return TTbuild_term(TTfunLLL_literal, args);
}

TTterm
TTbuildLLL_ident(TTtype_string t)
{
    args[0] = (TTterm)t;
    return TTbuild_term(TTfunLLL_ident, args);
}

BOOL
TTmatchLLL_alternation(TTterm t, TTterm *t1, TTterm *t2, TTterm *t3, TTterm *t4, TTterm *t5)
{
    if (t->fun == TTfunLLL_alternation)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        *t3 = t->u.args[2];
        *t4 = t->u.args[3];
        *t5 = t->u.args[4];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchLLL_basis(TTterm t, TTterm *t1)
{
    if (t->fun == TTfunLLL_basis)
    {
        *t1 = t->u.args[0];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchLLL_conjunction(TTterm t, TTterm *t1)
{
    if (t->fun == TTfunLLL_conjunction)
    {
        *t1 = t->u.args[0];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchLLL_disjunction(TTterm t, TTterm *t1)
{
    if (t->fun == TTfunLLL_disjunction)
    {
        *t1 = t->u.args[0];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchLLL_group(TTterm t, TTterm *t1, TTterm *t2, TTterm *t3)
{
    if (t->fun == TTfunLLL_group)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        *t3 = t->u.args[2];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchLLL_repetition(TTterm t, TTterm *t1)
{
    if (t->fun == TTfunLLL_repetition)
    {
        *t1 = t->u.args[0];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchLLL_rule(TTterm t, TTterm *t1, TTterm *t2, TTterm *t3, TTterm *t4)
{
    if (t->fun == TTfunLLL_rule)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        *t3 = t->u.args[2];
        *t4 = t->u.args[3];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchLLL_specification(TTterm t, TTterm *t1)
{
    if (t->fun == TTfunLLL_specification)
    {
        *t1 = t->u.args[0];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchLLL_term(TTterm t, TTterm *t1, TTterm *t2)
{
    if (t->fun == TTfunLLL_term)
    {
        *t1 = t->u.args[0];
        *t2 = t->u.args[1];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchLLL_literal(TTterm t, TTtype_string *t1)
{
    if (t->fun == TTfunLLL_literal)
    {
        *t1 = t->u.args[0];
        return TRUE;
    }
    return FALSE;
}

BOOL
TTmatchLLL_ident(TTterm t, TTtype_string *t1)
{
    if (t->fun == TTfunLLL_ident)
    {
        *t1 = t->u.args[0];
        return TRUE;
    }
    return FALSE;
}

