
#include <stdio.h>
#include <assert.h>
#include <sys/types.h>

#include "ttterm.h"
#include "ttbuffer.h"
#include "ttutils.h"

#include "LLLParse.h"
#include "lllutils.h"
#include "lllpp.h"

#define MAX_LINE_SIZE 1024

extern int snprintf(char *s,  size_t  n,  const  char  *format, ...);

static TTbuf st_buf;

static void dumpIdent(TTterm id)
{
	const char * p;
	TTtype_string str;

	assert(TTmatchLLL_ident(id, &str));
	assert(TTmatch_string(str, &p));
	TTbuffer_append(st_buf, p);
}

static BOOL dump(TTterm t);

void LLLpretty_rule(TTbuf buf, TTterm t)
{
	TTterm hd, id, der, dis, dot, xs;
	BOOL first;

	st_buf = buf;

	assert(TTmatchLLL_rule(t, &id, &der, &dis, &dot));
	dumpIdent(id);
	first = TRUE;
	assert(TTmatchLLL_disjunction(dis, &xs));
	while (TTmatch_cons(xs, &hd, &xs)) {
		if (first) {
			TTbuffer_append(st_buf, "\n  :");
			first = FALSE;
		} else {
			TTbuffer_append(st_buf, "\n  |");
		}
		TTtraverse(hd, dump);
	}
	TTbuffer_append(st_buf, "\n  ;");
}

static BOOL dump(TTterm t)
{
	TTterm basis, rep, rep_opt, x, hd, tl;
	const char * str;

	if (TTmatch_cons(t, &hd, &tl) &&
	    hd->fun == TTfunLLL_rule) {
		TTtraverse(hd, dump);
		if (!TTmatch_nil(tl)) {
			TTbuffer_append(st_buf, "\n\n");
			TTtraverse(tl, dump);
		}
		return TRUE;
	} else if (t->fun == TTfunLLL_rule) {
		LLLpretty_rule(st_buf, t);
		return TRUE;
	} else if (TTmatchLLL_term(t, &basis, &rep_opt)) {
		TTtraverse(basis, dump);
		if (TTmatch_singleton(rep_opt, &rep)) {
			assert(TTmatchLLL_repetition(rep, &x));
			assert(TTmatch_string(x, &str));
			TTbuffer_append(st_buf, str);
		}
		return TRUE;
	} else if (TTmatch_string(t, &str)) {
		TTbuffer_append(st_buf, " ");
		TTbuffer_append(st_buf, str);
		return TRUE;
	}
	return FALSE;
}

void LLLpretty(TTbuf buf, TTterm t)
{
	st_buf = buf;

	TTtraverse(t, dump);
}

void LLLpretty_with_header(TTbuf buf, TTterm spec)
{
	static char line[MAX_LINE_SIZE];

	snprintf(line, MAX_LINE_SIZE, "# Idents:      %d\n", TTlength(LLLget_idents(spec)));
	TTbuffer_append(buf, line);
	snprintf(line, MAX_LINE_SIZE, "# - top:       %d\n", TTlength(LLLget_top_idents(spec)));
	TTbuffer_append(buf, line);
	snprintf(line, MAX_LINE_SIZE, "# - used:      %d\n", TTlength(LLLget_used_idents(spec)));
	TTbuffer_append(buf, line);
	snprintf(line, MAX_LINE_SIZE, "# - defined:   %d\n", TTlength(LLLget_def_idents(spec)));
	TTbuffer_append(buf, line);
	snprintf(line, MAX_LINE_SIZE, "# - undefined: %d\n", TTlength(LLLget_undef_idents(spec)));
	TTbuffer_append(buf, line);
	snprintf(line, MAX_LINE_SIZE, "# Literals:    %d\n", TTlength(LLLget_literals(spec)));
	TTbuffer_append(buf, line);
	TTbuffer_append(buf, "\n");
	LLLpretty(buf, spec);
	TTbuffer_append(buf, "\n");
}

