
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>

#include "ttterm.h"
#include "ttbuffer.h"
#include "ttutils.h"
#include "ttparse.h"

#include "SDFScan.h"
#include "SDFParse.h"

#define MAX_KW_SIZE 100

TTterm TTparseSDF_LAYOUT(void)
{
	const char *buf = TTparse_get_string();
	size_t len = 0;

	while ((buf[len] <= ' ' && buf[len] != '\0')
	    || buf[len] == '%'
	    || (buf[len] == '/' && buf[len+1] == '/')) {
		while (buf[len] <= ' ' && buf[len] != '\0') {
			len++;
		}
		while (buf[len] == '%') {
			len++;
			len += strcspn(buf + len, "\n");
		}
		while (buf[len] == '/' && buf[len+1] == '/') {
			len += 2;
		}
	}

	return TTparse_word(len);
}

static TTterm scan_word(size_t len)
{
	TTterm t1, t2;

	t1 = TTparse_word(len);
	t2 = TTparseSDF_LAYOUT();
	return TTbuild_sandwich(TTbuild_empty_string(), t1, t2);
}

TTterm TTparseSDF_KEYWORD(const char * name)
{
	const char *buf;
	size_t len = 0;
	BOOL is_keyword = FALSE;

	buf = TTparse_get_string();

	while (name[len] != '\0' && buf[len] != '\0') {
		if (name[len] != buf[len])
			return 0;
		if (isalnum((int) name[len]))
			is_keyword = TRUE;
		len++;
	}
	if (len == 0)
		return 0;
	if (is_keyword && (isalnum((int) buf[len])))
		return 0;
	return scan_word(len);
}

TTterm TTparseSDF_literal( void )
{
/* ["][^"\n]*["] */
	const char *buf;
	size_t len;

	buf = TTparse_get_string();

	len = 0;
	if (buf[len] == '\"') {
		len++;
		len += strcspn(buf + len, "\"");
		if (buf[len] == '\"') {
			len++;
			return TTbuildSDF_literal(scan_word(len));
		}
	}
	return 0;
}

TTterm TTparseSDF_ident( void )
{
	const char *buf;
	size_t len;
	static char tmp[MAX_KW_SIZE + 1];

	buf = TTparse_get_string();

	if (!isalpha((int) buf[0]))
		return 0;

	len = 1;
	while (isalnum((int) buf[len]) || buf[len] == '-'
	       || buf[len] == '_') {
		len++;
	}
	if (len < MAX_KW_SIZE) {
		strncpy(tmp, buf, len);
		tmp[len] = '\0';
		if (SDFisKeyword(tmp))
			return 0;
	}
	return TTbuildSDF_ident(scan_word(len));
}

TTterm TTparseSDF_char_class( void )
{
/* [\[][^\]\n]*[\]] */
	const char *buf;
	size_t len;

	buf = TTparse_get_string();

	len = 0;
	if (buf[len] == '~')
		len++;
	if (buf[len] == '[') {
		len++;
		len += strcspn(buf + len, "]");
		if (buf[len] == ']') {
			len++;
			return TTbuildSDF_char_class(scan_word(len));
		}
	}
	return 0;
}

TTterm TTparseSDF_DISJUNCTION(TTfun fun, size_t n, TTterm(**parsers) (void))
{
	return TTparse_disjunction(fun, n, parsers);
}

TTterm TTparseSDF_CONJUNCTION(TTfun fun, TTterm(**parsers) (void))
{
	return TTparse_conjunction(fun, parsers);
}
