
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include "ttbuffer.h"
#include "tterror.h"
#include "ttfile.h"
#include "gc.h"

#define BUFSZ 1024

static char st_tmp_buf[BUFSZ];

extern int snprintf(char *s,  size_t  n,  const  char  *format, ...);

TTbuf TTbuffer_create(void)
{
	TTbuf buf = (TTbuf) GC_malloc(sizeof(struct TTbuf_struct));
	buf->name = "";
	buf->alloc_size = 16;
	buf->data = (char *)GC_malloc(buf->alloc_size);
	/* Make sure Flex can handle this buffer as is. */
	buf->data[0] = '\0';
	buf->data[1] = '\0';
	buf->size = 0;

	return buf;
}

void TTbuffer_destroy(TTbuf buf)
{
	GC_free(buf->data);
	GC_free(buf);
}

static void resize(TTbuf buf, int size)
{
	if (size + 2 > buf->alloc_size) {
		buf->alloc_size = size * 2 + 2;
		buf->data = (char *) GC_realloc(buf->data, buf->alloc_size);
	}
	buf->size = size;
	/* Make sure Flex can handle this buffer as is. */
	buf->data[buf->size] = '\0';
	buf->data[buf->size+1] = '\0';
}

void TTbuffer_read(TTbuf buf, FILE * f)
{
	size_t len;

	buf->name = "standard input";

	resize(buf, 0);
	while ((len = fread(st_tmp_buf, 1, BUFSZ-1, f)) != 0) {
		st_tmp_buf[len] = '\0';
		TTbuffer_append(buf, st_tmp_buf);
	}
}

void TTbuffer_read_named(TTbuf buf, const char * name)
{
	FILE * f = TTfile_open(name, "r");
	TTbuffer_read(buf, f);
	buf->name = name;
	fclose(f);
}

void TTbuffer_write(TTbuf buf, FILE * f)
{
	fwrite(buf->data, 1, buf->size, f);
}

void TTbuffer_write_named(TTbuf buf, const char * name)
{
	FILE * f = TTfile_open(name, "w");
	TTbuffer_write(buf, f);
	fclose(f);
}

void TTbuffer_append(TTbuf buf, const char * str)
{
	int pos = buf->size;

	resize(buf, (int)(buf->size + strlen(str)));
	strcpy(buf->data + pos, str);
}

void TTbuffer_get_location(TTbuf buf, int cur, int * line, int * col)
{
	int ch;
	size_t i = 0;

	*line = 1;
	*col = 1;

	while (i < cur) {
		ch = buf->data[i++];
		if (ch == '\n') {
			(*line)++;
			*col = 1;
		} else {
			(*col)++;
		}
	}
}

