
TTfun TTfun_cons;
TTfun TTfun_nil;
TTfun TTfun_tuple;
TTfun TTfun_sandwich;
/*TTfun TTfun_anno;*/
TTfun TTfun_layout;

typedef TTterm TTtype_tuple;
typedef TTterm TTtype_sandwich;
/*typedef TTterm TTtype_anno;*/
typedef TTterm TTtype_layout;
typedef TTterm TTtype_list;
typedef TTterm TTtype_fm;

typedef TTterm (*TTtrafo_f)(TTterm);
typedef TTterm (*TTtrafoenv_f)(TTterm, TTterm);
typedef BOOL (*TTtrav_f)(TTterm);
typedef void (*TTtrav_line_f)(TTterm, int, int);
typedef TTterm (*TTmap_f)(TTterm);
typedef void (*TTmapvoid_f)(TTterm);
typedef BOOL (*TTfilter_f)(TTterm);

void            TTutils_init( void );

/* Tuple functions. */
TTtype_tuple	TTbuild_tuple(TTterm t1, TTterm t2);
BOOL            TTmatch_tuple(TTtype_tuple t, TTterm *t1, TTterm *t2);
BOOL		TTis_tuple(TTterm t);

/* List functions. */
int             TTlength(TTtype_list t);
TTtype_list	TTconcat(TTtype_list x, TTtype_list y);
TTtype_list	TTbuild_cons(TTterm head, TTtype_list tail);
TTtype_list	TTbuild_nil( void );
BOOL            TTis_list(TTterm t);
BOOL            TTmatch_cons(TTtype_list t, TTterm *head, TTtype_list *tail);
BOOL            TTmatch_nil(TTtype_list t);
TTterm		TThead(TTtype_list t);
TTtype_list	TTtail(TTtype_list t);
TTtype_list	TTrepeat(int n, TTterm x);
TTtype_list	TTlinit(TTtype_list xs);
TTtype_list	TTbuild_singleton(TTterm x);
BOOL            TTmatch_singleton(TTtype_list x, TTterm *y);
TTtype_list	TTreverse(TTtype_list xs);
TTtype_list	TTtake(int n, TTtype_list xs);
TTtype_list	TTdrop(int n, TTtype_list xs);

/* Higher order functions on lists. */
TTtype_list	TTmap(TTmap_f f, TTtype_list xs);
void            TTmap_void(TTmapvoid_f f, TTtype_list xs);
TTtype_list	TTfilter(TTfilter_f, TTtype_list xs);
TTtype_list	TTfilter_map(TTmap_f f, TTtype_list t);

/* Set functions. */
BOOL            TTelem(TTterm x, TTtype_list xs);
BOOL		TTsubset(TTtype_list xs, TTtype_list ys);
TTtype_list	TTunion(TTtype_list xs, TTtype_list ys);
TTtype_list	TTdiff(TTtype_list xs, TTtype_list ys);

/* FiniteMap functions. */
TTtype_fm	TTempty_fm( void );
TTtype_fm	TTadd_to_fm(TTtype_fm fm, TTterm key, TTterm elt);
TTterm		TTlookup_fm(TTtype_fm fm, TTterm key);

/* String functions. */
size_t		TTstrlen(TTtype_string t);
void            TTdump_string(FILE *f, TTtype_string t);
TTtype_string	TTunquote(TTtype_string t);
TTtype_string	TTquote(int open, int close, TTtype_string t);
TTtype_string	TTto_upper(TTtype_string t);
TTtype_string	TTcapitalize(TTtype_string t);
TTtype_string	TTreplace(int c1, int c2, TTtype_string t);
BOOL            TTequal_string(TTtype_string t, const char * name);
BOOL		TTequal_string_ci(TTtype_string t, const char *name);
TTtype_string	TTbuild_empty_string(void);
TTtype_string	TTconcat_string(TTtype_string x, TTtype_string y);
TTtype_string	TTbuild_string_n(const char * str, size_t len);

/* Traversal functions. */
TTterm		TTtransform(TTterm t, TTtrafo_f f);
TTterm		TTtransform_nonstop(TTterm t, TTreduce_f f);
TTterm		TTanalyze(TTterm t,
			  TTtrafo_f f,
			  TTterm (*comb)(TTterm, TTterm),
                          TTterm empty);
void		TTtraverse(TTterm t, TTtrav_f f);
void		TTtraverse_line(TTterm t, TTtrav_line_f f, int line, int col);
TTterm		TTselect(TTterm t, TTtrafo_f f);

/* I/O functions. */
void	TTunparse(TTbuf buf, TTterm t);
void	TTshow(FILE *f, TTterm t);

/* Special functions to deal with layout in syntax trees. */
TTtype_sandwich	TTbuild_sandwich(TTterm t1, TTterm t2, TTterm t3);
BOOL            TTmatch_sandwich(TTtype_sandwich t, TTterm *t1, TTterm *t2, TTterm *t3);
TTterm		TTunsandwich(TTterm t);
TTterm		TTunsandwich_rec(TTterm t);
BOOL		TTutils_equal(TTterm t1, TTterm t2);

/* Annotations, meant to replace sandwich as a more general concept. */
/* Not yet, this concept has to be explored more first. */
/*
TTtype_anno	TTbuild_anno(TTterm t1, TTterm t2);
BOOL		TTmatch_anno(TTtype_anno t, TTterm *t1, TTterm *t2);
TTterm		TTunanno(TTterm t);
TTterm		TTunanno_rec(TTterm t);
BOOL		TTequal_ast(TTterm t1, TTterm t2);
*/

/* Shallow parameterized type checking. */
BOOL TTtypecheck_optional(TTterm t, BOOL (*f)(TTterm));
BOOL TTtypecheck_plus(TTterm t, BOOL (*f)(TTterm));
BOOL TTtypecheck_star(TTterm t, BOOL (*f)(TTterm));
BOOL TTtypecheck_alternation(TTterm t, BOOL (*f1)(TTterm), BOOL (*f2)(TTterm));
BOOL TTtypecheck_keyword(TTterm t, const char * str);
BOOL TTtypecheck_literal(TTterm t, const char * str);

/* The weight of the term: 1 + the weight of its children. */
int TTweight(TTterm t);

