
#include <stdio.h>
#include <stdlib.h>
#include "ttterm.h"
#include "ttstrat.h"
#include "ttstratp.h"
#include "ttbuffer.h"
#include "ttutils.h"

#define try TTbuild_try
#define totaltd TTbuild_totaltd
#define adhoc TTbuild_adhoc
#define succeed TTbuild_succeed
#define innermost TTbuild_innermost
#define naive_innermost TTbuild_naive_innermost

static TTterm incTP(TTterm t)
{
	int n;

	if (TTmatch_int(t, &n)) {
		return TTbuild_int(n+1);
	}
	return 0;
}

static TTterm incTU(TTterm t)
{
	int n;

	if (TTmatch_int(t, &n)) {
		return TTbuild_singleton(TTbuild_int(n+1));
	}
	return 0;
}

static TTterm from_to(int i, int j)
{
	if (i > j) {
		return TTbuild_nil();
	}
	return TTbuild_cons(TTbuild_int(i), from_to(i+1, j));
}

static void demo_stratTP(void)
{
	TTterm s, t, tp;

	/* Create a strategy s and a term t and show them. */
	s = totaltd(try(adhoc(incTP)));
	t = TTbuild_cons(from_to(0, 1),
	    TTbuild_cons(from_to(2, 3),
	    TTbuild_nil()));
	printf("----- Demo for type preserving strategies.\n");
	printf("Original term t    = ");
	TTshow(stdout, t);
	printf("\n");
	printf("Strategy s         = ");
	printf("totaltd(try(adhoc(incTP)))\n");

	/* Apply the strategy. */
	tp = TTstrat_apply(s, t, 0);
	printf("apply(s, t, 0)     = ");
	TTshow(stdout, tp);
	printf("\n");
}

static void demo_stratTU(void)
{
	TTterm s, t, tu;
	struct TTstrat_env_struct env;

	/* Create a strategy s and a term t and show them. */
	s = totaltd(try(adhoc(incTU)));
	t = TTbuild_cons(from_to(0, 1),
	    TTbuild_cons(from_to(2, 3),
	    TTbuild_nil()));
	printf("----- Demo for type unifying strategies.\n");
	printf("Original term t    = ");
	TTshow(stdout, t);
	printf("\n");
	printf("Strategy s         = ");
	printf("totaltd(try(adhoc(incTU)))\n");

	/* Apply the strategy. */
	env.comb = TTconcat;
	env.singleton = TTbuild_singleton;
	env.empty = TTbuild_nil();
	tu = TTstrat_apply(s, t, &env);
	printf("apply(s, t, env)   = ");
	TTshow(stdout, tu);
	printf("\n");
}

int
main(int argc, char **argv)
{
	TTstrat_init();
	TTutils_init();

	demo_stratTP();
	demo_stratTU();

	return 0;
}



